import fileinput
import sys

MODULE_FILE_PATH = "/usr/local/lib/python3.10/dist-packages/tensorrt_llm/hlapi/utils.py"


def patch():
    search_text = "signal.signal(signal.SIGINT, sigint_handler)"

    with fileinput.FileInput(MODULE_FILE_PATH, inplace=True) as file:
        for line in file:
            if search_text in line:
                line = "    # " + line.lstrip()
            sys.stdout.write(line)



import logging
import subprocess
import time
from pathlib import Path
from typing import Optional

from helpers.context_managers import current_directory
from shared.util import kill_child_processes

INFERENCE_SERVER_FAILED_FILE = Path("~/inference_server_crashed.txt").expanduser()
TERMINATION_TIMEOUT_SECS = 120.0
TERMINATION_CHECK_INTERVAL_SECS = 0.5


class InferenceServerProcessController:
    _inference_server_process: Optional[subprocess.Popen] = None
    _inference_server_port: int
    _inference_server_home: str
    _app_logger: logging.Logger
    _inference_server_process_args: list[str]
    _logged_unrecoverable_since_last_restart: bool

    def __init__(
        self,
        inference_server_home: str,
        inference_server_process_args: list[str],
        inference_server_port: int,
        app_logger: logging.Logger,
    ) -> None:
        self._inference_server_home = inference_server_home
        self._inference_server_process_args = inference_server_process_args
        self._inference_server_port = inference_server_port
        self._inference_server_started = False
        self._inference_server_ever_started = False
        self._inference_server_terminated = False
        self._logged_unrecoverable_since_last_restart = False
        self._app_logger = app_logger

    def start(self, inf_env: dict):
        with current_directory(self._inference_server_home):
            inf_env["INFERENCE_SERVER_PORT"] = str(self._inference_server_port)
            self._inference_server_process = subprocess.Popen(
                self._inference_server_process_args, env=inf_env
            )

            self._inference_server_started = True
            self._inference_server_ever_started = True
            self._logged_unrecoverable_since_last_restart = False

    def _terminate_children_and_process(self):
        """Kill child processes first, then parent. Prevents port binding conflicts."""
        # Use a shorter timeout than the truss patch read timeout (=120s):
        # see remote/baseten/api.py:_post_graphql_query()
        kill_child_processes(self._inference_server_process.pid, timeout_seconds=30)
        self._inference_server_process.terminate()

    def stop(self):
        if self._inference_server_process is not None:
            self._terminate_children_and_process()
            self._inference_server_process.wait()

        self._inference_server_started = False

    def terminate_with_wait(self):
        self._terminate_children_and_process()
        self._inference_server_terminated = True
        termination_check_attempts = int(
            TERMINATION_TIMEOUT_SECS / TERMINATION_CHECK_INTERVAL_SECS
        )
        for _ in range(termination_check_attempts):
            time.sleep(TERMINATION_CHECK_INTERVAL_SECS)
            # None returncode means alive
            # https://docs.python.org/3.9/library/subprocess.html#subprocess.Popen.returncode
            if self._inference_server_process.poll() is not None:
                return

    def inference_server_started(self) -> bool:
        return self._inference_server_started

    def inference_server_ever_started(self) -> bool:
        return self._inference_server_ever_started

    def is_inference_server_running(self) -> bool:
        # Explicitly check if inference server process is up, this is a bit expensive.
        if not self._inference_server_started:
            return False

        if self._inference_server_process is None:
            return False

        return self._inference_server_process.poll() is None

    def is_inference_server_intentionally_stopped(self) -> bool:
        return INFERENCE_SERVER_FAILED_FILE.exists()

    def is_inference_server_terminated(self) -> bool:
        return self._inference_server_terminated

    def check_and_recover_inference_server(self, inf_env: dict):
        if (
            self.inference_server_started()
            and not self.is_inference_server_running()
            and not self.is_inference_server_terminated()
        ):
            if not self.is_inference_server_intentionally_stopped():
                self._app_logger.warning(
                    "Inference server seems to have crashed, restarting"
                )
                self.start(inf_env)
            else:
                if not self._logged_unrecoverable_since_last_restart:
                    self._app_logger.warning(
                        "Inference server unrecoverable. Try patching"
                    )
                    self._logged_unrecoverable_since_last_restart = True



from unittest.mock import MagicMock

from truss.remote.baseten import service
from truss.remote.baseten.core import ModelVersionHandle


def test_model_invoke_url_prod():
    url = service.URLConfig.invoke_url(
        "https://model-123.api.baseten.co",
        service.URLConfig.MODEL,
        "789",
        is_draft=False,
    )
    assert url == "https://model-123.api.baseten.co/deployment/789/predict"


def test_model_invoke_url_draft():
    url = service.URLConfig.invoke_url(
        "https://model-123.api.baseten.co",
        service.URLConfig.MODEL,
        "789",
        is_draft=True,
    )
    assert url == "https://model-123.api.baseten.co/development/predict"


def test_chain_invoke_url_prod():
    url = service.URLConfig.invoke_url(
        "https://chain-abc.api.baseten.co",
        service.URLConfig.CHAIN,
        "666",
        is_draft=False,
    )
    assert url == "https://chain-abc.api.baseten.co/deployment/666/run_remote"


def test_chain_invoke_url_draft():
    url = service.URLConfig.invoke_url(
        "https://chain-abc.api.baseten.co",
        service.URLConfig.CHAIN,
        "666",
        is_draft=True,
    )
    assert url == "https://chain-abc.api.baseten.co/development/run_remote"


def test_model_status_page_url():
    url = service.URLConfig.status_page_url(
        "https://app.baseten.co", service.URLConfig.MODEL, "123"
    )
    assert url == "https://app.baseten.co/models/123/overview"


def test_chain_status_page_url():
    url = service.URLConfig.status_page_url(
        "https://app.baseten.co", service.URLConfig.CHAIN, "abc"
    )
    assert url == "https://app.baseten.co/chains/abc/overview"


def test_model_logs_url():
    url = service.URLConfig.model_logs_url("https://app.baseten.co", "123", "789")
    assert url == "https://app.baseten.co/models/123/logs/789"


def test_chain_logs_url():
    url = service.URLConfig.chainlet_logs_url(
        "https://app.baseten.co", "abc", "666", "543"
    )
    assert url == "https://app.baseten.co/chains/abc/logs/666/543"


def test_predict_response_to_json():
    """Test that predict method returns JSON response for normal dict result."""
    # Create a mock BasetenService
    mock_handle = MagicMock(spec=ModelVersionHandle)
    mock_handle.model_id = "test-model"
    mock_handle.version_id = "test-version"
    mock_handle.hostname = "https://model-test.api.baseten.co"

    mock_api = MagicMock()
    mock_api.app_url = "https://app.baseten.co"

    service_instance = service.BasetenService(
        model_version_handle=mock_handle,
        is_draft=False,
        api_key="test-key",
        service_url="https://test.com",
        api=mock_api,
    )

    # Mock the _send_request method to return a successful JSON response
    mock_response = MagicMock()
    mock_response.json.return_value = {"result": "success"}
    service_instance._send_request = MagicMock(return_value=mock_response)

    # Test predict method
    result = service_instance.predict({"input": "test"})

    # Verify that the JSON response is returned directly
    assert result == {"result": "success"}

    # Test non-dict response types below

    # With integer response
    mock_response.json.return_value = 42
    result = service_instance.predict({"input": "test"})
    assert result == 42

    # With string response
    mock_response.json.return_value = "success"
    result = service_instance.predict({"input": "test"})
    assert result == "success"

    # With list response
    mock_response.json.return_value = [1, 2, 3, 4]
    result = service_instance.predict({"input": "test"})
    assert result == [1, 2, 3, 4]

    # With boolean response
    mock_response.json.return_value = True
    result = service_instance.predict({"input": "test"})
    assert result is True



import re
from dataclasses import dataclass
from typing import Optional
from urllib.parse import parse_qs, urlparse

from packaging import requirements  # type: ignore

URL_PATTERN = re.compile(r"^(https?|git|svn|hg|bzr)(\+|:\/\/)")


def is_url_based_requirement(req: str) -> bool:
    return bool(URL_PATTERN.match(req.strip()))


def get_egg_tag(req: str) -> Optional[list[str]]:
    if is_url_based_requirement(req):
        parsed_url = urlparse(req)
        fragments = parse_qs(parsed_url.fragment)
        return fragments.get("egg", None)
    return None


def reqs_by_name(reqs: list[str]) -> dict[str, str]:
    return {identify_requirement_name(req): req for req in reqs if req.strip()}


def identify_requirement_name(req: str) -> str:
    try:
        # parse as a url requirement
        req = req.strip()
        if is_url_based_requirement(req):
            parsed_url = urlparse(req)
            # Identify the package by it's unique components. We can't reliably id a package name
            # via URL, so we fallback on this instead.
            vcs_options = ["git", "svn", "hg", "bzr"]
            for vcs in vcs_options:
                if req.startswith(vcs + "+"):
                    return f"{vcs}+{parsed_url.netloc}{parsed_url.path.split('@')[0]}"

            return req

        parsed_req = requirements.Requirement(req)
        return parsed_req.name  # type: ignore
    except (requirements.InvalidRequirement, ValueError):
        # default to the whole line if we can't parse it.
        return req.strip()


@dataclass
class RequirementMeta:
    requirement: str
    name: str
    is_url_based_requirement: bool
    egg_tag: Optional[list[str]]

    def __hash__(self):
        return hash(self.name)

    @staticmethod
    def from_req(req: str) -> "RequirementMeta":
        return RequirementMeta(
            req,
            identify_requirement_name(req),
            is_url_based_requirement(req),
            get_egg_tag(req),
        )
